package cz.frantovo.nekurak.ejb;

import cz.frantovo.nekurak.dao.PodnikDAO;
import cz.frantovo.nekurak.dto.Komentar;
import cz.frantovo.nekurak.dto.Podnik;
import cz.frantovo.nekurak.dto.VysledekHlasovani;
import cz.frantovo.nekurak.ext.Geo;
import cz.frantovo.nekurak.ext.Geo.Souradnice;
import cz.frantovo.nekurak.ext.Texy;
import cz.frantovo.nekurak.util.Komentare;
import cz.frantovo.nekurak.vyjimky.KomentarovaVyjimka;
import cz.frantovo.nekurak.vyjimky.TexyVyjimka;
import java.util.Collection;
import java.util.logging.Logger;
import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;

/**
 *
 * @author fiki
 */
@Stateless
public class PodnikEJB implements PodnikRemote {

	private static final Logger log = Logger.getLogger(PodnikEJB.class.getSimpleName());
	@EJB
	private PodnikDAO podnikDAO;
	@Resource
	private SessionContext ctx;

	public Collection<Podnik> getPodniky() {
		Collection<Podnik> vysledek = podnikDAO.getPodniky();
		return vysledek;
	}

	public Podnik getPodnik(int id) {
		return podnikDAO.getPodnik(id);
	}

	@RolesAllowed("opravneny")
	public void zalozPodnik(Podnik p) {
		p.setSpravce(ctx.getCallerPrincipal().getName());
		podnikDAO.zaloz(p);
	}

	@RolesAllowed("opravneny")
	public void upravPodnik(Podnik p) {
		podnikDAO.uloz(p);
	}

	public int dopocitejSouradnice() {
		/**
		 * TODO: refaktorovat, změnit datové typy, souřadnice…
		 */
		Geo g = new Geo();
		int pocetAktualizovanych = 0;

		for (Podnik p : podnikDAO.getPodnikyBezSouradnic()) {
			Souradnice s = g.getSouradnice(p.getUlice() + " " + p.getCisloPopisne() + ", " + p.getMesto());
			if (s != null) {
				pocetAktualizovanych++;
				p.setSirka(s.getSirka());
				p.setDelka(s.getDelka());
				podnikDAO.uloz(p);
			}
		}

		return pocetAktualizovanych;
	}

	public void hlasuj(int podnik, boolean hlas, String ipAdresa) {

		podnikDAO.hlasuj(podnik, hlas, ipAdresa);
	}

	public VysledekHlasovani getVysledekHlasovani(int podnik) {
		return podnikDAO.getVysledekHlasovani(podnik);
	}

	private void kontrolaKomentare(Komentar k) throws TexyVyjimka, KomentarovaVyjimka {
		k.setUzivatel(ctx.getCallerPrincipal().getName());
		k.setDatum(null);

		if (k.getKomentar() == null || k.getKomentar().length() < 1) {
			throw new KomentarovaVyjimka("Nechceme prázdné komentáře.", null);
		}

		/** Převedeme na XML */
		switch (k.getTyp()) {
			case PROSTY_TEXT:
				k.setKomentar(Komentare.upravProstyText(k.getKomentar()));
				break;
			case TEXY:
				Texy t = new Texy();
				k.setKomentar(t.preved(k.getKomentar()));
				break;
			case XHTML:
				k.setKomentar(Komentare.upravXHTML(k.getKomentar()));
				break;
		}
		/** Přidáme kořenový element */
		k.setKomentar(Komentare.obal(k.getKomentar()));

		/** Zkontrolujeme XML */
		Komentare.zkontroluj(k.getKomentar());
	}

	@RolesAllowed("opravneny")
	public Komentar komentuj(Komentar k, boolean uloz) throws KomentarovaVyjimka, TexyVyjimka {
		kontrolaKomentare(k);
		if (uloz) {
			podnikDAO.zaloz(k);
		}
		return k;
	}
}
