package cz.frantovo.nekurak.rest;

import cz.frantovo.nekurak.dto.Clanek;
import cz.frantovo.nekurak.web.HledacSluzby;
import cz.frantovo.nekurak.xml.ClanekXML;
import java.util.ArrayList;
import java.util.Collection;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;

@Path("clanek")
public class ClankyREST {

	private static final String MIME_XML = "text/xml";
	private static final String MIME_TEXT = "text/plain";
	private HledacSluzby hledac = new HledacSluzby();

	/** Vypíšeme seznam všech článků v systému */
	@GET
	@Path("/")
	@Produces(MIME_XML)
	public Collection<ClanekXML> seznam() {
		Collection<ClanekXML> vysledek = new ArrayList<ClanekXML>();
		Collection<Clanek> clanky = hledac.getClanekEJB().getClanky();

		for (Clanek c : clanky) {
			vysledek.add(new ClanekXML(c));
		}

		return vysledek;
	}

	/** Získáme konkrétní článek */
	@GET
	@Path("/{id}")
	@Produces(MIME_XML)
	public ClanekXML ziskej(@PathParam("id") int id) {
		Clanek c = hledac.getClanekEJB().getClanek(id);
		return new ClanekXML(c);
	}

	/**
	 * Vložíme nový článek
	 * @return ID založeného článku (návratový typ musí být String – ne int)
	 */
	@POST
	@Consumes(MIME_XML)
	@Produces(MIME_TEXT)
	public String zaloz(ClanekXML xml) {
		int id = hledac.getClanekEJB().zalozClanek(xml.getClanek());
		return id + "\n";
	}

	/** Aktualizujeme článek */
	@PUT
	@Consumes(MIME_XML)
	@Path("/{id}")
	public void uprav(@PathParam("id") int id, ClanekXML xml) {
		xml.setId(id);
		hledac.getClanekEJB().upravClanek(xml.getClanek());
	}

	/** Smažeme článek */
	@DELETE
	@Path("/{id}")
	public void smaz(@PathParam("id") int id) {
		Clanek c = new Clanek();
		c.setId(id);
		hledac.getClanekEJB().smazClanek(c);
	}
}
